import matplotlib.pyplot as plt
from matplotlib import cm
from matplotlib.colors import ListedColormap

import numpy as np

def plot_overlay_segmentation(x, y, save_path=None, spacing=(1,1), step=1,):

    true = np.array(np.argmax(y,axis=-1), dtype='float32')
    true[true == 0] = np.nan

    # turn axis
    x_ = x

    # extract color from colormap
    nb_class = np.nanmax(true)
    col = []
    dark2 = cm.get_cmap("Dark2")
    for i in range(int(nb_class)):
        col.append(dark2(i))
    col = ListedColormap(col)

    asp = spacing[0]/spacing[1]
    
    for i in range(0, x.shape[0], step):
    # ground truth and prediction
        f, axarr = plt.subplots(1, 2)
        axarr[0].imshow(x_[i, :, :, 0], 'gray', interpolation='none', aspect=asp)
        axarr[0].set_title("MRI")
        axarr[0].axis('off')

        axarr[1].imshow(x_[i, :, :, 0], 'gray', interpolation='none', aspect=asp)
        axarr[1].imshow(true[i, :, :], col, interpolation='none', alpha=0.3, aspect=asp)
        axarr[1].set_title("Ground Truth")
        axarr[1].axis('off')

              
 #       f.savefig(save_path + '_' + str(i) + '.png', dpi=300, quality=95, frameon=False, bbox_inches='tight', padding=0)
 #       plt.close(f)

def plot_compare_segmentation(x, y_true, y_pred, save_path=None, spacing=(1,1), step=1):

    pred = np.array(np.argmax(y_pred,axis=-1), dtype='float32')
    true = np.array(np.argmax(y_true,axis=-1), dtype='float32')

    pred[pred == 0] = np.nan
    true[true == 0] = np.nan

    x_ = x

    # extract color from colormap
    nb_class = np.nanmax(true)
    col = []
    dark2 = cm.get_cmap("Dark2")
    for i in range(int(nb_class)):
        col.append(dark2(i))
    col = ListedColormap(col)

    asp = spacing[0]/spacing[1]
    
    for i in range(0, x.shape[0], step):
    # ground truth and prediction
        f, axarr = plt.subplots(1, 3)
        axarr[0].imshow(x_[i, :, :, 0], 'gray', interpolation='none', aspect=asp)
        axarr[0].set_title("MRI")
        axarr[0].axis('off')
        
        axarr[1].imshow(x_[i, :, :, 0], 'gray', interpolation='none', aspect=asp)
        axarr[1].imshow(true[i, :, :], col, interpolation='none', alpha=0.3, aspect=asp)
        axarr[1].set_title("Ground Truth")
        axarr[1].axis('off')

        axarr[2].imshow(x_[i, :, :, 0], 'gray', interpolation='none', aspect = asp)
        axarr[2].imshow(pred[i, :, :], col, interpolation='none', alpha=0.3, aspect=asp)
        axarr[2].set_title('Prediction')
        axarr[2].axis('off')

 #       f.savefig(save_path + '_' + str(i) + '.png', dpi=300, quality=95, frameon=False, bbox_inches='tight', padding=0)
 #       plt.close(f)

    
    
def plot_segm_history(history, metrics=['iou', 'val_iou'], losses=['loss', 'val_loss']):
    # summarize history for iou
    plt.figure(figsize=(12,6))
    for metric in metrics:
        plt.plot(history.history[metric], linewidth=3)
    plt.suptitle('metrics over epochs', fontsize=20)
    plt.ylabel('metric', fontsize=20)
    plt.xlabel('epoch', fontsize=20)
    #plt.yticks(np.arange(0.3, 1, step=0.02), fontsize=35)
    #plt.xticks(fontsize=35)
    plt.legend(metrics, loc='center right', fontsize=15)
    plt.show()
    # summarize history for loss
    plt.figure(figsize=(12,6))    
    for loss in losses:
        plt.plot(history.history[loss], linewidth=3)
    plt.suptitle('loss over epochs', fontsize=20)
    plt.ylabel('loss', fontsize=20)
    plt.xlabel('epoch', fontsize=20)
    #plt.yticks(np.arange(0, 0.2, step=0.005), fontsize=35)
    #plt.xticks(fontsize=35)
    plt.legend(losses, loc='center right', fontsize=15)
    plt.show()
